@@ -109,18 +109,21 @@ inline double giac_log(double d){
 extern "C" int ctrl_c_interrupted(int exception);
 extern "C" void console_print(const char * s);
 extern "C" const char * console_prompt(const char * s);
+int giac_readFileSize(const char * buffer,size_t header_offset) ;
+int tar_filesize(int s);
 
 bool dfu_get_scriptstore_addr(size_t & start,size_t & taille,char & altdfu);
-bool dfu_get_scriptstore(const char * fname);
+int dfu_get_scriptstore(const char * fname); // returns size
 bool dfu_send_scriptstore(const char * fname);
 bool dfu_send_rescue(const char * fname);
 bool dfu_send_bootloader(const char * fname);
-const int nwstoresize1=0x8000,nwstoresize2=0x8014;
 // send to 0x90000000+offset*0x10000
 bool dfu_send_firmware(const char * fname,int offset);
 bool dfu_send_apps(const char * fname);
-bool dfu_send_slotab(const char * fnamea,const char * fnameb1,const char * fnameb2);
+bool dfu_send_slotab(const char * fnamea1,const char * fnamea2,const char * fnameb1,const char * fnameb2);
 bool dfu_update_khicas(const char * fname); 
+int nwstore_skip_sys(const unsigned char * ptr,int nwstoresize);
+
 
 #if defined HAVE_LIBMICROPYTHON
 #include <string>
@@ -402,6 +405,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern int ALG_EXT_DIGITS;
   // 0 use a unique algebraic extension, >0: don't search a unique alg ext if degree is >
   extern int TRY_FU_UPRIME;
+  extern int TRY_FU_UPRIME_MAXLEAFSIZE;
   extern int SOLVER_MAX_ITERATE;
   extern int MAX_PRINTABLE_ZINT;
   extern int MAX_RECURSION_LEVEL;
@@ -434,6 +438,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
 #endif
   extern double powlog2float;
   extern int MPZ_MAXLOG2;
+  extern int SET_COMPARE_MAXIDNT;
 
 #ifdef WITH_MYOSTREAM
   // replacement for std::cerr
@@ -586,7 +591,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   typedef void (* giac_callback)(const giac::gen & ,void * );
 
   struct thread_param {
-    bool _kill_thread;
+    int _kill_thread;
     int thread_eval_status;
     giac_callback f;
     void * f_param;
@@ -633,7 +638,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   typedef std::map<std::string,nwsrec,ltstring> nws_map;
   std::string dos2unix(const std::string & res);
   bool scriptstore2map(const char * fname,nws_map & m);
-  bool map2scriptstore(const nws_map & m,const char * fname);
+  bool map2scriptstore(const nws_map & m,const char * fname,int nwstoresize);
   // check that filename content matches a file content signed in sigfilename
   bool sha256_check(const char * sigfilename,const char * filename);
   bool nws_certify_firmware(bool with_overwrite,GIAC_CONTEXT); // Numworks certification
@@ -1202,8 +1207,8 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   // Check if a thread_eval is active
   bool is_context_busy(GIAC_CONTEXT);
   // Check and set the kill thread flag
-  bool kill_thread(GIAC_CONTEXT);
-  void kill_thread(bool b,GIAC_CONTEXT);
+  int kill_thread(GIAC_CONTEXT);
+  void kill_thread(int b,GIAC_CONTEXT);
   // Thread eval status = 0 finished, =1 eval, =2 debug_wait_main
   int thread_eval_status(GIAC_CONTEXT);
   void thread_eval_status(int c,GIAC_CONTEXT);
