@@ -3,16 +3,31 @@
 #define _KDISPLAY_H
 #include "config.h"
 #include "giacPCH.h"
-#ifdef KHICAS
+extern "C" void copy_to_xcas_clipboard(const char * s);
+extern "C" const char * get_xcas_clipboard();
+#if defined KHICAS || defined SDL_KHICAS
 #include "misc.h"
 
 #include <exception>
+
+#if defined TICE && !defined std
+#define std ustl
+#endif
+#if defined FX || defined FXCG
+typedef unsigned char Char;
+#else
+typedef char Char;
+#endif
+
+extern bool xthetat;
+
 #ifdef NUMWORKS
 extern char * freeptr;
 extern const char * flash_buf;
 extern "C" const char * flash_read(const char * filename);
 extern "C" int flash_filebrowser(const char ** filenames,int maxrecords,const char * extension);
 #endif
+
 class autoshutdown : public std::exception{
   const char * what () const throw ()
   {
@@ -20,10 +35,19 @@ class autoshutdown : public std::exception{
   }
 };
 
+#ifndef LCD_WIDTH_PX
 extern  const int LCD_WIDTH_PX;
 extern   const int LCD_HEIGHT_PX;
+#endif
+
 extern char* fmenu_cfg;
-#define color_gris 57083
+#ifdef HP39
+#define color_gris 0
+#define MENUHEIGHT 8
+#else
+#define color_gris 46491
+#define MENUHEIGHT 12
+#endif
 #define STATUS_AREA_PX 0 // 24
 #define GIAC_HISTORY_MAX_TAILLE 32
 #define GIAC_HISTORY_SIZE 2
@@ -38,6 +62,7 @@ extern "C" {
   void mp_stack_ctrl_init();
   extern int parser_errorline,parser_errorcol;
   void python_free();
+  extern bool nws_freezeturtle;
 }
 int micropy_ck_eval(const char *line);
 #endif
@@ -74,6 +99,8 @@ extern "C" {
   double r_det(double *,int);
   struct double_pair {
     double r,i;
+    double_pair operator +=(const double_pair &);
+    double_pair operator -=(const double_pair &);
   } ;
   typedef struct double_pair c_complex;
   bool matrice2c_complexptr(const giac::matrice &M,c_complex *x);
@@ -98,25 +125,52 @@ extern "C" {
   void c_turtle_up(int i);
   void c_turtle_goto(double x,double y);
   void c_turtle_cap(double x);
-  void c_turtle_crayon(int i);
+  int c_turtle_crayon(int i);
   void c_turtle_rond(int x,int y,int z);
   void c_turtle_disque(int x,int y,int z,int centered);
   void c_turtle_fill(int i);
   void c_turtle_fillcolor(double r,double g,double b,int entier);
   void c_turtle_getposition(double * x,double * y);
+  void c_turtle_clear(int clrpos);
+  void c_turtle_show(int visible);
+  int c_turtle_getcap();
+  void c_turtle_towards(double x,double y);
+  int c_turtle_getcolor();
+  void c_turtle_color(int);
+  void c_turtle_fillcolor1(int c);
+  extern int shell_x,shell_y,shell_fontw,shell_fonth; 
+  
 }
+#ifdef NUMWORKS_SLOTBFR
+extern const int lang;
+#else
 extern int lang;
+#endif
 extern short int nspirelua;
 extern bool warn_nr;
+
 int select_interpreter(); // 0 Xcas, 1|2 Xcas python_compat(1|2), 3 MicroPython, 4 QuickJS 
 const char * gettext(const char * s) ;
+#ifdef HP39
+int kcas_main(int isAppli, unsigned short OptionNum);
+extern giac::context * contextptr; 
+#endif
+
+#ifdef BW
+int get_free_memory();
+int PrintMini(int x,int y,const char * s,int mode);
+#endif
 
 #ifndef NO_NAMESPACE_XCAS
 namespace xcas {
 #endif // ndef NO_NAMESPACE_XCAS
   void set_exam_mode(int i,const giac::context *);
   int giac_filebrowser(char * filename,const char * extension,const char * title,int storage=0);
+#ifdef BW
+  inline void draw_rectangle(int x,int y,int w,int h,int c){os_fill_rect(x,y,w,h,c);}
+#else
   void draw_rectangle(int x,int y,int w,int h,int c);
+#endif
   void draw_line(int x0,int y0,int x1,int y1,int c);
   void draw_circle(int xc,int yc,int r,int color,bool q1=true,bool q2=true,bool q3=true,bool q4=true);
   void draw_filled_circle(int xc,int yc,int r,int color,bool left=true,bool right=true);
@@ -138,6 +192,7 @@ namespace xcas {
   void Equation_select(giac::gen & eql,bool select);
   int eqw_select_down(giac::gen & g);
   int eqw_select_up(giac::gen & g);
+  giac::gen Equation_copy(const giac::gen & g);
 
   giac::gen Equation_compute_size(const giac::gen & g,const giac::attributs & a,int windowhsize,const giac::context * contextptr);
   giac::eqwdata Equation_total_size(const giac::gen & g);  
@@ -168,7 +223,7 @@ namespace xcas {
   typedef struct
   {
     std::string s;
-    color_t color=giac::_BLACK;
+    color_t color=::giac::_BLACK;
     short int newLine=0; // if 1, new line will be drawn before the text
     short int spaceAtEnd=0;
     short int lineSpacing=0;
@@ -231,6 +286,9 @@ namespace xcas {
     int4(int u_,int d_,int du_,int dd_):u(u_),d(d_),du(du_),dd(dd_) {}
   };
   
+  struct int4bis {
+    int u,d,du,dd;
+  };
   // quaternion struct for more intuitive rotations
   struct quaternion_double {
     double w,x,y,z;
@@ -253,6 +311,7 @@ namespace xcas {
     int2(int i_,int j_):i(i_),j(j_) {}
     int2(): i(0),j(0) {}
   };
+
   inline bool operator < (const int2 & a,const int2 & b){ if (a.i!=b.i) return a.i<b.i; return a.j<b.j;}
   inline bool operator == (const int2 & a,const int2 & b){ return a.i==b.i && a.j==b.j;}
 
@@ -363,6 +422,7 @@ namespace xcas {
     void set_gen_value(int n,const giac::gen & g,bool exec=true); // set n-th entry value
     int geo_handle(int event,int key);
     int ui();
+    void curve_infos();
     void init_tracemode();
     giac::vecteur selected_names(bool allobjects,bool withdef) const;
     void find_title_plot(giac::gen & title_tmp,giac::gen & plot_tmp);
@@ -423,8 +483,9 @@ namespace xcas {
 
   void save_session(const giac::context * );
 #if 1
-#define MAX_FILENAME_SIZE 63
-  void save_console_state_smem(const char * filename,bool xwaspy,const giac::context *);
+#define MAX_FILENAME_SIZE 270
+  bool QRdisp(const char * text,const char *msg);
+  void save_console_state_smem(const char * filename,bool xwaspy,bool qr,const giac::context *);
   bool load_console_state_smem(const char * filename,const giac::context *);
 
   struct DISPBOX {
@@ -435,7 +496,6 @@ namespace xcas {
     unsigned char mode;
   } ;
 
-
   enum CONSOLE_RETURN_VAL {
 			   CONSOLE_NEW_LINE_SET = 1,
 			   CONSOLE_SUCCEEDED = 0,
@@ -465,16 +525,25 @@ namespace xcas {
 		    UPPER_CASE
   };
 
-  enum CONSOLE_SCREEN_SPEC {
+  enum CONSOLE_SCREEN_SPEC
+    {			    
 #ifdef NSPIRE_NEWLIB
-			    _LINE_MAX = 128,
-			    COL_DISP_MAX = 32,
+     _LINE_MAX = 128,
+     COL_DISP_MAX = 32,
 #else
-			    _LINE_MAX = 48,
-			    COL_DISP_MAX = 30,//32
+     _LINE_MAX = 48,
+#ifdef NUMWORKS
+     COL_DISP_MAX = 30,//32
+#else // HP39
+     COL_DISP_MAX = 35,//21  //!!!!!!! 21     
 #endif
-			    LINE_DISP_MAX = 11,
-			    EDIT_LINE_MAX = 2048
+#endif
+#ifdef HP39
+     LINE_DISP_MAX = 7,      //!!!!!!!  7
+#else     
+     LINE_DISP_MAX = 11,
+#endif
+     EDIT_LINE_MAX = 2048,
   };
   
   struct console_line {
@@ -497,10 +566,16 @@ namespace xcas {
   };
 
 #define MAX_FMENU_ITEMS 8
+#ifdef HP39
+#define FMENU_TITLE_LENGHT 8
+#else
 #define FMENU_TITLE_LENGHT 4
-
+#endif
+  
 #define is_wchar(c) ((c == 0x7F) || (c == 0xF7) || (c == 0xF9) || (c == 0xE5) || (c == 0xE6) || (c == 0xE7))
+#ifndef HP39
 #define printf(s) Console_Output((const char *)s);
+#endif
 
   int Console_DelStr(char *str, int end_pos, int n);
   int Console_InsStr(char *dest, const char *src, int disp_pos);
@@ -526,7 +601,9 @@ namespace xcas {
   char* Console_GetEditLine();
   void dConsolePut(const char *);
   void dConsolePutChar(const char );
+#ifndef BW
   void dConsoleRedraw(void);
+#endif
   extern int dconsole_mode;
   extern int console_changed; // 1 if something new in history
   extern char session_filename[MAX_FILENAME_SIZE+1];
@@ -566,6 +643,10 @@ namespace xcas {
 } // namespace xcas
 #endif // ndef NO_NAMESPACE_XCAS
 
+#ifdef BW
+void dConsoleRedraw(void);
+#endif
+
 giac::gen sheet(const giac::context *); // in kadd.cc
 /* ************************************************************
 **************************************************************
@@ -595,12 +676,12 @@ namespace giac {
     int value:4=MENUITEM_VALUE_NONE; // value of the menu item. For example, if type is MENUITEM_CHECKBOX and the checkbox is checked, the value of this var will be MENUITEM_VALUE_CHECKED
     int isselected:4=0; // for file browsers and other multi-select screens, this will show an arrow before the item
     short int isfolder=0; // for file browsers, this will signal the item is a folder
-    signed char color=giac::_BLACK; // color of the menu item (use TEXT_COLOR_* to define)
+    signed char color=::giac::_BLACK; // color of the menu item (use TEXT_COLOR_* to define)
     // The following two settings require the menu type to be set to MENUTYPE_MULTISELECT
 #if 0
     signed char icon=-1; //for file browsers, to show a file icon. -1 shows no icon (default)
 #endif
-    MenuItem():token(0),type(MENUITEM_NORMAL),value(MENUITEM_VALUE_NONE),isselected(0),isfolder(0),color(giac::_BLACK) {}
+    MenuItem():token(0),type(MENUITEM_NORMAL),value(MENUITEM_VALUE_NONE),isselected(0),isfolder(0),color(::giac::_BLACK) {}
   } ;
 
   typedef struct
@@ -618,12 +699,16 @@ namespace giac {
     char* statusText = NULL; // text to be shown on the status bar, may be empty
     char* title = NULL; // title to be shown on the first line if not null
     char* subtitle = NULL;
-    int titleColor=giac::_BLUE; //color of the title
+    int titleColor=::giac::_BLUE; //color of the title
     char* nodatamsg; // message to show when there are no menu items to display
     int startX=1; //X where to start drawing the menu. NOTE this is not absolute pixel coordinates but rather character coordinates
     int startY=0; //Y where to start drawing the menu. NOTE this is not absolute pixel coordinates but rather character coordinates
     int width=30; // NOTE this is not absolute pixel coordinates but rather character coordinates
+#ifdef HP39
+    int height=8;
+#else
     int height=12; // NOTE this is not absolute pixel coordinates but rather character coordinates
+#endif
     int scrollbar=1; // 1 to show scrollbar, 0 to not show it.
     int scrollout=0; // whether the scrollbar goes out of the menu area (1) or it overlaps some of the menu area (0)
     int numitems; // number of items in menu
@@ -659,13 +744,17 @@ namespace giac {
   const char * keytostring(int key,int keyflag,bool py,const giac::context * contextptr);
   void insert(std::string & s,int pos,const char * add);
   
-  int showCatalog(char* insertText,int preselect,int menupos,const giac::context * contextptr);
+  int showCatalog(char* insertText,int preselect,int menupos=0,const giac::context * contextptr=0);
   int doMenu(Menu* menu, MenuItemIcon* icontable=NULL);
   void reset_alpha();
   // category=0 for CATALOG, 1 for OPTN
   // returns 0 on exit, 1 on success
   int doCatalogMenu(char* insertText, const char* title, int category,const giac::context * contextptr);
+#ifdef NUMWORKS_SLOTB
+#define shortcuts_fr_string shortcuts_en_string
+#else
   extern const char shortcuts_fr_string[];
+#endif
   extern const char shortcuts_en_string[];
   extern const char apropos_fr_string[];
   extern const char apropos_en_string[];
@@ -673,38 +762,48 @@ namespace giac {
 
   gen _efface_logo(const gen & g,GIAC_CONTEXT);
   gen turtle_state(const giac::context * contextptr);
-  int inputline(const char * msg1,const char * msg2,std::string & s,bool numeric,int ypos=65,const giac::context *contextptr=0);
+  int inputline(const char * msg1,const char * msg2,std::string & s,bool numeric,int ypos,const giac::context *contextptr);
   extern logo_turtle * turtleptr;
   bool inputdouble(const char * msg1,double & d,const giac::context *contextptr);
+#ifndef BW
   bool do_confirm(const char * s);
   int confirm(const char * msg1,const char * msg2,bool acexit=false,int y=40);
   bool confirm_overwrite();
   void invalid_varname();
+#endif
 
 #ifndef NO_NAMESPACE_XCAS
 } // namespace giac
 #endif // ndef NO_NAMESPACE_XCAS
 
-
-#define COLOR_BLACK giac::_BLACK
-#define COLOR_RED giac::_RED
-#define COLOR_GREEN giac::_GREEN
-#define COLOR_CYAN giac::_CYAN
-#define COLOR_BLUE giac::_BLUE
-#define COLOR_YELLOW giac::_YELLOW
-#define COLOR_MAGENTA giac::_MAGENTA
-#define COLOR_WHITE giac::_WHITE
+#ifndef BW
+#ifdef HP39
+#define COLOR_CYAN   90
+#define COLOR_RED    68
+#define COLOR_GREEN  68
+#define COLOR_WHITE  255
+#define COLOR_BLACK  0
+#else
+#define COLOR_BLACK ::giac::_BLACK
+#define COLOR_RED ::giac::_RED
+#define COLOR_GREEN ::giac::_GREEN
+#define COLOR_CYAN ::giac::_CYAN
+#define COLOR_WHITE ::giac::_WHITE
+#endif
+#define COLOR_BLUE ::giac::_BLUE
+#define COLOR_YELLOW ::giac::_YELLOW
+#define COLOR_MAGENTA ::giac::_MAGENTA
 #define COLOR_YELLOWDARK 64934
 #define COLOR_BROWN 65000
-#define TEXT_COLOR_BLACK giac::_BLACK
-#define TEXT_COLOR_RED giac::_RED
-#define TEXT_COLOR_GREEN giac::_GREEN
-#define TEXT_COLOR_CYAN giac::_CYAN
-#define TEXT_COLOR_BLUE giac::_BLUE
-#define TEXT_COLOR_YELLOW giac::_YELLOW
-#define TEXT_COLOR_WHITE giac::_WHITE
-#define TEXT_COLOR_MAGENTA giac::_MAGENTA
-
+#define TEXT_COLOR_BLACK ::giac::_BLACK
+#define TEXT_COLOR_RED ::giac::_RED
+#define TEXT_COLOR_GREEN ::giac::_GREEN
+#define TEXT_COLOR_CYAN ::giac::_CYAN
+#define TEXT_COLOR_BLUE ::giac::_BLUE
+#define TEXT_COLOR_YELLOW ::giac::_YELLOW
+#define TEXT_COLOR_WHITE ::giac::_WHITE
+#define TEXT_COLOR_MAGENTA ::giac::_MAGENTA
+#endif
 
 #endif // _KDISPLAY_H
 #endif
