@@ -10,9 +10,12 @@ var UI = {
   langue: -1, // -1 french
   calc: 2, // 1 KhiCAS, 2 Numworks, 3 TI Nspire CX
   calculator:0, // !=0 if hardware Numworks connected
+  appsaddress:0x90200000,
   calculator_connected:false,
   tar_first_modif_offset:0,
-  nws_connect:function(){
+  current_interface:0,
+  nws_connect:function(intf=0){
+    UI.current_interface=intf;
     UI.nws_progress=$id('plog');
     UI.nws_progresslegend=$id('ploglegend');
     if (navigator.usb){
@@ -100,12 +103,12 @@ var UI = {
     for (;i<l;++i){
       let cur=sig[i];
       if (cur[0]!=fname) continue;
-      console.log('sig_check',i,cur[1],data.byteLength);
+      console.log('sigfile_index',i,'sig_length',cur[1],'calcdata_length',data.byteLength);
       if (cur[1]>data.byteLength) continue;
       let dat=data.slice(0,cur[1]);
       let digest = await window.crypto.subtle.digest('SHA-256', dat);
       digest=Array.from(new Uint8Array(digest));
-      console.log('signature file sha256',cur[2],'data sha256',digest);
+      console.log('signature file sha256',cur[2],'calcdata sha256',digest);
       let j=0;
       for (;j<32;++j){
 	let tst=(digest[j]-cur[2][j]) % 256;
@@ -138,6 +141,7 @@ var UI = {
   backup2rec:function(buf){
     let pos=4,nwstoresize=32768;
     let view=new Uint8Array(buf);
+    nwstoresize=view.length;
     let rec=[];
     for (;pos<nwstoresize;){
       let L=view[pos+1]*256+view[pos];
@@ -164,12 +168,38 @@ var UI = {
 	code=new TextDecoder().decode(new Uint8Array(buf,pos+i+2,len-1));
       let b=new Blob([data], {type: 'application/octet-binary'});
       let record={"name":name, "type":type,  "autoImport": false, "data": b,"code":code,"size": len}
-      //console.log(record);
+      console.log('backup2rec',rec.length,record);
       rec.push(record)
       pos += L;
     }
     return rec;
   },
+should_addsys: async function(file) {
+    let storage = await UI.calculator.backupStorage();
+    let newrec = [storage.records[0], storage.records[1]];
+
+    // On attend directement la lecture ici
+    const arrayBuffer = await file.arrayBuffer();
+    
+    // On continue seulement quand c'est fini
+    UI.storage_records = UI.backup2rec(arrayBuffer);
+    
+    let rec = UI.storage_records;
+    console.log("Lecture terminée, rec est prêt :", rec);
+    if (rec.length>=2 && rec[0].name=="pr" && rec[1].nam=="gp")
+      return 0; // pr and gp already there
+    for (let j=0;j<rec.length;++j)
+      newrec.push(rec[j]);
+    console.log('adding pr.sys and gp.sys, other records',rec.length,newrec.length);
+    storage.records=newrec;
+    console.log(newrec);
+    console.log(storage);
+    await UI.calculator.installStorage(storage, function() {
+      // Do stuff after writing to the storage is done
+      console.log('backup with added header saved to Numworks');
+    });
+    return 1;
+  },
   numworks_retry:function(){
     alert(UI.langue==-1?'Calculatrice detectee. Reessayez!':'Calculator detected! Try again');
   },
@@ -184,8 +214,9 @@ var UI = {
       let pinfo = await UI.calculator.getPlatformInfo();
         
       let storage_blob = await UI.calculator.__retreiveStorage(pinfo["storage"]["address"], pinfo["storage"]["size"]);
+      console.log('storage size',pinfo["storage"]["size"]);
       filename = "backup.nws";
-      saveAs(storage_blob.slice(0,32768), filename);
+      saveAs(storage_blob.slice(0,pinfo["storage"]["size"]), filename);
       return;
     }
     let storage = await UI.calculator.backupStorage();
@@ -226,7 +257,7 @@ var UI = {
       let data=evt.target.result;
       if (!backup)
 	data+=String.fromCharCode(0);
-      UI.numworks_save(file.name,data);
+      UI.numworks_save(file,data);
     }
   },
   download:function(url, filename) {
@@ -240,13 +271,14 @@ var UI = {
       })
       .catch(console.error);
   },
-  numworks_save:function(filename,S){
+  numworks_save:function(file,S){
     //console.log(filename,S); return;
-    UI.nws_connect();
+    UI.nws_connect(1);
     //window.setTimeout(UI.numworks_save_,100,filename,S);
-    window.setTimeout(UI.numworks_save_,100,filename,S);
+    window.setTimeout(UI.numworks_save_,100,file,S);
   },
-  numworks_save_:async function(filename,S){
+  numworks_save_:async function(file,S){
+    let filename=file.name;
     if (UI.calculator==0 || !UI.calculator_connected){
       alert(UI.langue==-1?UI.chkmsgfr:chkmsgen);
       if (UI.calculator) UI.calculator.stopAutoConnect();
@@ -255,6 +287,9 @@ var UI = {
     if (filename.length>4 && filename.substr(filename.length-4,4)==".nws"){
       let pinfo = await UI.calculator.getPlatformInfo();
       UI.calculator.device.startAddress = pinfo["storage"]["address"];
+      console.log('numworks_save storage size',pinfo["storage"]["size"]);
+      if (pinfo["storage"]["size"]>32768 && UI.should_addsys(file))
+        return 0;
       //console.log(UI.calculator.device.startAddress,S); return;
       let res=await UI.calculator.device.do_download(UI.calculator.transferSize, S, false);
       if (res==0)
@@ -341,7 +376,7 @@ var UI = {
     if (tar_only==1
         || (!version_longue && confirm(UI.langue==-1?"Installer les scripts et sessions exemples?":"Install script and example sessions?"))
        ){
-      UI.calculator.device.startAddress = 0x90200000;
+      UI.appsaddress = UI.calculator.device.startAddress = 0x90200000;
       UI.calculator.device.logProgress(0,'KhiCAS script and examples at 0x90200000');
       let data=await UI.loadfile('khi92.tar');
       let res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
@@ -399,7 +434,7 @@ var UI = {
     if (!khionly){
       UI.print('erase/write KhiCAS apps, wait about 2 minutes');
       data=await UI.loadfile('apps.tar');
-      UI.calculator.device.startAddress = 0x90200000;
+      UI.appsaddress = UI.calculator.device.startAddress = 0x90200000;
       UI.calculator.device.logProgress(0,'apps');
       res=await UI.calculator.device.do_download(UI.calculator.transferSize, data, false);
       UI.print(res?'KhiCAS installed':'error installing KhiCAS');
@@ -1173,7 +1208,7 @@ await UI.calculator.device.device_.controlTransferOut({
     console.log(filename,finfo);
     //return;
     if (sendcalc){
-      UI.calculator.device.startAddress = 0x90200000;
+      UI.calculator.device.startAddress = 0x90200000; // UI.appsaddress;
       UI.calculator.device.logProgress(0,'apps');
       let res=await UI.calculator.device.do_download(UI.calculator.transferSize, resbuffer, false);
       alert(UI.langue==-1?'Installation terminee. Appuyer sur RESET a l\'arriere de la calculatrice':'Install success. Press RESET on the calculator back.');
@@ -1207,13 +1242,14 @@ await UI.calculator.device.device_.controlTransferOut({
       for (let i=0;i<size;++i)
 	buf[i]=src[start+i];
       let buffer=buf.buffer;
-      console.log('sendtar',UI.tar_first_modif_offset,start,size,buffer.byteLength);
-      UI.calculator.device.startAddress = 0x90200000+start;
+      console.log('sendtar appstart 1st_modif start size buflength',UI.appsaddress.toString(16),UI.tar_first_modif_offset.toString(16),start.toString(16),size.toString(16),buffer.byteLength.toString(16));
+      UI.calculator.device.startAddress = UI.appsaddress+start;
       UI.calculator.device.logProgress(0,'apps');
+      return;
       res=await UI.calculator.device.do_download(UI.calculator.transferSize, buffer, false);      
     }
     else {
-      UI.calculator.device.startAddress = 0x90200000;
+      UI.calculator.device.startAddress = UI.appsaddress;//0x90200000; // 
       UI.calculator.device.logProgress(0,'apps');
       res=await UI.calculator.device.do_download(UI.calculator.transferSize, UI.numworks_buffer, false);
     }
